#include <fstream>
#include <iostream>
#include "AneuMeshLoader.h"
#include "Exceptions.h"

void AneuMeshLoader::LoadMesh(const std::string& filename) {
  std::ifstream myfile(filename);
  if (!myfile) {
    throw NoFileFoundException(filename);
  } else std::cout << "File \"" << filename << "\" was opened successfully." << std::endl;

  int amount, dimension;

  myfile >> amount >> dimension;
  nodes.reserve(amount);

  for (int i = 1; i <= amount; ++i) {
    Node temp_node;
    temp_node.id = i;
    myfile >> temp_node.coord[0] >> temp_node.coord[1] >> temp_node.coord[2];
    temp_node.top = true;
    nodes.push_back(temp_node);
  }

  myfile >> amount >> dimension;
  elements.reserve(amount);

  for (int i = 1; i <= amount; ++i) {
    FiniteElement temp_element;
    temp_element.element_id = i;
    myfile >> temp_element.material_id;
    temp_element.nodes_id.reserve(dimension);
    int temp_id;
    for (int j = 0; j < dimension; ++j) {
      myfile >> temp_id;
      temp_element.nodes_id.push_back(temp_id);
    }
    elements.push_back(std::move(temp_element));
  }

  myfile >> amount >> dimension;
  boundary.reserve(amount);

  for (int i = 1; i <= amount; ++i) {
    BoundaryFiniteElement temp_surface;
    temp_surface.element_id = i;
    myfile >> temp_surface.border_id;
    temp_surface.nodes_id.reserve(dimension);
    int temp_id;
    for (int j = 0; j < dimension; ++j) {
      myfile >> temp_id;
      temp_surface.nodes_id.push_back(temp_id);
    }
    boundary.push_back(std::move(temp_surface));
  }

  myfile.close();
  std::cout << "All data was loaded, closing the file..." << std::endl;
}