#pragma once
#include <iostream>
#include <vector>
#include <array>
#include <fstream>

struct Node {
  int id;
  std::array<double, 3> coord;

  bool top;

  Node() = default;
  Node(int, std::array<double, 3>, bool);

  bool operator==(const Node&) const;
  bool operator<(const Node&) const;

  friend std::ostream& operator<<(std::ostream&, const Node&);
  friend std::ostream& operator<<(std::ostream&, const std::vector<Node>&);
};

struct FiniteElement {
  int element_id;
  int material_id;
  std::vector<int> nodes_id;

  friend std::ostream& operator<<(std::ostream&, const FiniteElement&);
  friend std::ostream& operator<<(std::ostream&, const std::vector<FiniteElement>&);
};

struct BoundaryFiniteElement {
  int element_id;
  int border_id;
  std::vector<int> nodes_id;

  friend std::ostream& operator<<(std::ostream&, const BoundaryFiniteElement&);
  friend std::ostream& operator<<(std::ostream&, const std::vector<BoundaryFiniteElement>&);
};

struct Edge {
  int first_node_id;
  int last_node_id;
  int center_id;

  Edge() = default;
  Edge(int, int, int);

  bool operator==(const Edge&) const;
  bool operator<(const Edge&) const;

  void UpdateCenter(int);
};


class EdgeHash {
public:
  std::size_t operator() (const Edge& _p) const;
};